/*
 * Support for creating a new process.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/simple_pager.h>

#include "process_creator_context_object_interface.h"



/* Forward declaration. */

class ProcessCreatorResource;



/* A process creating context employing a simple pager to transfer process
   details. */

class ProcessCreatorContextResource : public SimplePager,
                                      public ProcessCreatorContextObject
{
protected:
  ProcessCreatorResource *_creator;

public:
  explicit ProcessCreatorContextResource(ProcessCreatorResource *creator);

  virtual ~ProcessCreatorContextResource();

  /* Server details. */

  ipc_server_default_config_type config();

  void *interface()
  { return static_cast<ProcessCreatorContextObject *>(this); }

  /* Process creator context interface methods. */

  virtual long start(int argc, l4_cap_idx_t writer, l4_cap_idx_t *process);

  /* Pager/dataspace methods. */
  
  long map(offset_t offset, map_address_t hot_spot, map_flags_t flags,
           l4_snd_fpage_t *region)
  { return SimplePager::map(offset, hot_spot, flags, region); }
  
  long info(offset_t *size, map_flags_t *flags)
  { return SimplePager::info(size, flags); }
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
